﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;

namespace DataCarouselFun
{
    /// <summary>
    /// Class encapsulating manipulation of group panel data
    /// </summary>
    public class GroupPanelsBusinessLogic
    {
        #region Fields

        private ObservableCollection<GroupPanelData> _groupPanelList;

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets/sets the group panel observable collection.
        /// </summary>
        public ObservableCollection<GroupPanelData> GroupPanelList
        {
            get 
            { 
                return _groupPanelList; 
            }
            set 
            { 
                _groupPanelList = value; 
            }
        }

        #endregion Properties

        #region Methods

        /// <summary>
        /// Create and add a <see cref="GroupPanelData"/> to the observable collection.
        /// </summary>
        /// <param name="panelTitle"></param>
        public void AddGroupPanel(string panelTitle)
        {
            GroupPanelData panelData = new GroupPanelData(panelTitle);
            AddGroupPanel(panelData);
        }

        /// <summary>
        ///  Add a <see cref="GroupPanelData"/> to the observable collection.
        /// </summary>
        /// <param name="panelData"></param>
        public void AddGroupPanel(GroupPanelData panelData)
        {
            if (_groupPanelList == null)
            {
                _groupPanelList = new ObservableCollection<GroupPanelData>();
            }

            _groupPanelList.Add(panelData);
        }

        /// <summary>
        /// Gets the group panel observable collection.
        /// </summary>
        /// <returns></returns>
        public ObservableCollection<GroupPanelData> GetGroupPanels()
        { 
            AddGroupPanel("Item Data");
            AddGroupPanel("Inventory Data");
            AddGroupPanel("Location Data");
            GroupPanelData parentData = new GroupPanelData("Misc Data");
            parentData.AddChild(new GroupPanelData("Site"));
            parentData.AddChild(new GroupPanelData("Pricing"));
            parentData.AddChild(new GroupPanelData("Image"));
            AddGroupPanel(parentData);
            return _groupPanelList;
        }

        #endregion Methods
    }

    /// <summary>
    /// Class encapsulating the group panel data.
    /// </summary>
    public class GroupPanelData : INotifyPropertyChanged
    {
        #region Fields

        private string _groupPanelTitle;
        private IList<GroupPanelData> _childGroupPanelList;        

        #endregion Fields

        #region Properties

        /// <summary>
        /// Gets/sets the Title.
        /// </summary>
        public string Title
        {
            get { return _groupPanelTitle; }
            set
            {
                if (_groupPanelTitle != value)
                {
                    _groupPanelTitle = value;
                    OnPropertyChanged("Title");
                }
            }
        }

        /// <summary>
        /// Gets/sets the collection of child group panels
        /// </summary>
        public IList<GroupPanelData> ChildGroupPanelList
        {
            get { return _childGroupPanelList; }
            set { _childGroupPanelList = value; }
        }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// Instantiate a <see cref="GroupPanelData"/> instance.
        /// </summary>
        /// <param name="title"></param>
        public GroupPanelData(string title)
        {
            Title = title;
        }


        /// <summary>
        /// Instantiate a <see cref="GroupPanelData"/> instance. 
        /// </summary>
        /// <param name="title"></param>
        /// <param name="childPanels"></param>
        public GroupPanelData(string title, IList<GroupPanelData> childPanels)
        {
            Title = title;
            ChildGroupPanelList = childPanels;
        }

        #endregion Constructors  
      
        #region Method

        /// <summary>
        /// Create and add the <see cref="GroupPanelData"/> as a child of this group panel data.
        /// </summary>
        /// <param name="panelData"></param>
        public void AddChild(string childTitle)
        {
            GroupPanelData childData = new GroupPanelData(childTitle);
            AddChild(childData);
        }

        /// <summary>
        /// Add the <see cref="GroupPanelData"/> as a child of this group panel data.
        /// </summary>
        /// <param name="panelData"></param>
        public void AddChild(GroupPanelData panelData)
        {
            if (_childGroupPanelList == null)
            {
                _childGroupPanelList = new List<GroupPanelData>();
            }
            _childGroupPanelList.Add(panelData);
        }

        #endregion Method

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        private void OnPropertyChanged(String info)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(info));
            }
        }
        #endregion INotifyPropertyChanged Members
    }

}
